


#include "stdafx.h"
#include "QrToBitmap.h"

#include "qrencode.h"


extern "C"
{







//--------------------------------------------------------------------------
// CreateQrBitmap()        creates a bitmap from a string
// ----------------
// Input: pszContent = string to be encoded
//        nVersion = size (height and width) of output bitmap to be created, 0 for automatic size
//        nEcLevel = error correction level, one of QRECLEVEL
//        nMode = encoding mode, one of QRMODE_xxx
//        bCaseSensitive = case sensitive in 8-bit text mode
//        rgbqBack = background color with their alpha values in rgbReserved
//        rgbqFore = foreground color with their alpha values in rgbReserved (active pixels)
// Return: handle of newly created bitmap. It must be deleted by the caller with DeleteObject()
//         NULL if an error occured

DECLDIR HBITMAP CreateQrBitmap (const char *pszContent, int nVersion, QRECLEVEL nEcLevel, QRMODE nMode, BOOL bCaseSensitive, RGBQUAD rgbqBack, RGBQUAD rgbqFore, int nZoomFactor)
  {
  AFX_MANAGE_STATE(AfxGetStaticModuleState());
  HBITMAP hResult=NULL;

  ASSERT(QRMODE_NUM   == QR_MODE_NUM  ); // enum mismatch ??
	ASSERT(QRMODE_AN    == QR_MODE_AN   ); 
	ASSERT(QRMODE_8     == QR_MODE_8    ); 
	ASSERT(QRMODE_KANJI == QR_MODE_KANJI); 
	ASSERT(QRMODE_ECI   == QR_MODE_ECI  ); 
	ASSERT(QRECLEVEL_L == QR_ECLEVEL_L); // enum mismatch ??
	ASSERT(QRECLEVEL_M == QR_ECLEVEL_M); 
	ASSERT(QRECLEVEL_Q == QR_ECLEVEL_Q); 
	ASSERT(QRECLEVEL_H == QR_ECLEVEL_H);

  ASSERT(nZoomFactor>=1);
  nZoomFactor=max(nZoomFactor,1);

	if(QRcode*pQRC=QRcode_encodeString(pszContent,nVersion,(QRecLevel)nEcLevel,(QRencodeMode)nMode,bCaseSensitive))
		{
    if(unsigned char *pSrcLine=pQRC->data)
      {
      int nDataSize=pQRC->width*nZoomFactor*pQRC->width*nZoomFactor*sizeof(RGBQUAD);
      if(RGBQUAD *pPixels=(RGBQUAD *)malloc(nDataSize))
        {
        ZeroMemory(pPixels,nDataSize);
        RGBQUAD *pDest=pPixels;
        for(int nY=0;nY<pQRC->width;++nY)
          {
          for(int nYZoom=0;nYZoom<nZoomFactor;++nYZoom)
            {
            unsigned char *pSrc=pSrcLine;
            for(int nX=0;nX<pQRC->width;++nX)
              {
              RGBQUAD rgbqSet;
              if((*pSrc)&0x01)
                rgbqSet=rgbqFore;
              else
                rgbqSet=rgbqBack;
              for(int nXZoom=0;nXZoom<nZoomFactor;++nXZoom)
                {
                *pDest=rgbqSet;
                ++pDest;
                }
              ++pSrc;
              }
            }
          pSrcLine+=pQRC->width;
          }
        CBitmap bmpResult;
        bmpResult.CreateBitmap(pQRC->width*nZoomFactor,pQRC->width*nZoomFactor,1,sizeof(RGBQUAD)*8,pPixels);
        hResult=(HBITMAP)bmpResult.Detach();
        free(pPixels);
        }
      }
    QRcode_free(pQRC);
    }
  return hResult;
  }




} // extern "C"
